@@ -31,11 +31,6 @@ extern "C" {
 #include <sys/resource.h>
 }
 
-// A bit of obfuscation for Gentoo's sake.
-// We *need* -Werror for stapconf to work correctly.
-// https://bugs.gentoo.org/show_bug.cgi?id=522908
-#define WERROR ("-W" "error")
-
 #define PATH_ALLOWED_CHARS "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789+,-./_"
 
 using namespace std;
@@ -244,7 +239,7 @@ compile_dyninst (systemtap_session& s)
     {
       "gcc", "--std=gnu99", s.translated_source, s.symbols_source, "-o", module,
       "-fvisibility=hidden", "-O2", "-I" + s.runtime_path, "-D__DYNINST__",
-      "-Wall", WERROR, "-Wno-unused", "-Wno-strict-aliasing",
+      "-Wall", "-Werror", "-Wno-unused", "-Wno-strict-aliasing",
       "-Wno-pointer-to-int-cast", "-Wno-int-to-pointer-cast",
       "-Wno-pragmas", "-Wno-pointer-sign", "-Wno-format",
       "-pthread", "-lrt", "-ldw", "-fPIC", "-shared",
@@ -316,7 +311,7 @@ compile_pass (systemtap_session& s)
     << "$(LINUXINCLUDE) $(_KBUILD_CFLAGS) $(CFLAGS_KERNEL) $(EXTRA_CFLAGS) "
     << "$(CFLAGS) -DKBUILD_BASENAME=\\\"" << s.module_name << "\\\" "
     << "-Wmissing-prototypes "  // GCC14 prep, PR31288
-    << WERROR << " -S -o /dev/null -xc " << endl;
+    << "-Werror" << " -S -o /dev/null -xc " << endl;
   o << "stap_check_build = $(shell " << superverbose << " if $(CHECK_BUILD) $(1) "
     << redirecterrors << " ; then echo \"$(2)\"; else echo \"$(3)\"; fi)" << endl;
 
@@ -400,6 +395,8 @@ compile_pass (systemtap_session& s)
   output_exportconf(s, o2, "__module_text_address", "STAPCONF_MODULE_TEXT_ADDRESS");
   output_exportconf(s, o2, "add_timer_on", "STAPCONF_ADD_TIMER_ON");
   output_autoconf(s, o, cs, "autoconf-514-panic.c", "STAPCONF_514_PANIC", NULL);
+  output_autoconf(s, o, cs, "autoconf-task_work_cancel_func.c", "STAPCONF_TASK_WORK_CANCEL_FUNC", NULL);
+  output_autoconf(s, o, cs, "autoconf-pr32194-uprobe-register-unregister.c", "STAPCONF_PR32194_UPROBE_REGISTER_UNREGISTER", NULL);
   
   output_dual_exportconf(s, o2, "probe_kernel_read", "probe_kernel_write", "STAPCONF_PROBE_KERNEL");
   output_autoconf(s, o, cs, "autoconf-hw_breakpoint_context.c",
@@ -506,11 +503,10 @@ compile_pass (systemtap_session& s)
 
   output_autoconf(s, o, cs, "autoconf-pagefault_disable.c", "STAPCONF_PAGEFAULT_DISABLE", NULL);
   output_exportconf(s, o2, "kallsyms_lookup_name", "STAPCONF_KALLSYMS_LOOKUP_NAME_EXPORTED");
+  output_autoconf(s, o, cs, "autoconf-kallsyms_6_3.c", "STAPCONF_KALLSYMS_6_3", NULL);
+  output_autoconf(s, o, cs, "autoconf-kallsyms_6_4.c", "STAPCONF_KALLSYMS_6_4", NULL);
   output_autoconf(s, o, cs, "autoconf-uidgid.c", "STAPCONF_LINUX_UIDGID_H", NULL);
   output_exportconf(s, o2, "sigset_from_compat", "STAPCONF_SIGSET_FROM_COMPAT_EXPORTED");
-  output_exportconf(s, o2, "vzalloc", "STAPCONF_VZALLOC");
-  output_exportconf(s, o2, "vzalloc_node", "STAPCONF_VZALLOC_NODE");
-  output_exportconf(s, o2, "vmalloc_node", "STAPCONF_VMALLOC_NODE");
 
   // RHBZ1233912 - s390 temporary workaround for non-atomic udelay()
   output_exportconf(s, o2, "udelay_simple", "STAPCONF_UDELAY_SIMPLE_EXPORTED");
@@ -559,6 +555,9 @@ compile_pass (systemtap_session& s)
   output_autoconf(s, o, cs, "autoconf-files_lookup_fd_raw.c",
                   "STAPCONF_FILES_LOOKUP_FD_RAW", NULL);
   output_autoconf(s, o, cs, "autoconf-task-state.c", "STAPCONF_TASK_STATE", NULL);
+
+  output_autoconf(s, o, cs, "autoconf-linux-unaligned-h.c", "STAPCONF_LINUX_UNALIGNED_H", NULL);
+
   
   // used by runtime/linux/netfilter.c
   output_exportconf(s, o2, "nf_register_hook", "STAPCONF_NF_REGISTER_HOOK");
@@ -624,7 +623,7 @@ compile_pass (systemtap_session& s)
   o << "EXTRA_CFLAGS += $(call cc-option,-fno-ipa-icf)" << endl;
 
   // Assumes linux 2.6 kbuild
-  o << "EXTRA_CFLAGS += -Wno-unused " << WERROR << endl;
+  o << "EXTRA_CFLAGS += -Wno-unused " << "-Werror" << endl;
   #if CHECK_POINTER_ARITH_PR5947
   o << "EXTRA_CFLAGS += -Wpointer-arith" << endl;
   #endif
@@ -1067,7 +1066,7 @@ make_tracequeries(systemtap_session& s, const map<string,string>& contents)
   string makefile(dir + "/Makefile");
   ofstream omf(makefile.c_str());
   // force debuginfo generation, and relax implicit functions
-  omf << "EXTRA_CFLAGS := -g -Wno-implicit-function-declaration " << WERROR << endl;
+  omf << "EXTRA_CFLAGS := -g -Wno-implicit-function-declaration " << "-Werror" << endl;
   // RHBZ 655231: later rhel6 kernels' module-signing kbuild logic breaks out-of-tree modules
   omf << "CONFIG_MODULE_SIG := n" << endl;
   // PR23488: need to override this kconfig, else we get no useful struct decls
