@@ -404,16 +404,6 @@ static void *_stp_kzalloc(size_t size)
   return _stp_kzalloc_gfp(size, STP_ALLOC_FLAGS);
 }
 
-#ifndef STAPCONF_VZALLOC
-static void *vzalloc(unsigned long size)
-{
-	void *ret = vmalloc(size);
-	if (ret)
-		memset(ret, 0, size);
-	return ret;
-}
-#endif
-
 static void *_stp_vzalloc(size_t size)
 {
 	void *ret;
@@ -438,24 +428,6 @@ static void *_stp_vzalloc(size_t size)
 	return ret;
 }
 
-
-#ifndef STAPCONF_VMALLOC_NODE
-static void *vmalloc_node(unsigned long size, int node __attribute__((unused)))
-{
-	return vmalloc(size);
-}
-#endif
-
-#ifndef STAPCONF_VZALLOC_NODE
-static void *vzalloc_node(unsigned long size, int node)
-{
-	void *ret = vmalloc_node(size, node);
-	if (ret)
-		memset(ret, 0, size);
-	return ret;
-}
-#endif
-
 static void *_stp_vzalloc_node(size_t size, int node)
 {
 	void *ret;
