@@ -1,5 +1,5 @@
 /* main header file
- * Copyright (C) 2005-2012 Red Hat Inc.
+ * Copyright (C) 2005-2018 Red Hat Inc.
  * Copyright (C) 2005-2006 Intel Corporation.
  *
  * This file is part of systemtap, and is free software.  You can
@@ -19,6 +19,8 @@ static struct context *_stp_runtime_entryfn_get_context(void);
 static void _stp_runtime_entryfn_put_context(struct context *);
 static struct context *_stp_runtime_get_context(void);
 
+#include "addr-map.h"
+
 #if defined(__KERNEL__)
 
 #include "linux/runtime.h"
@@ -29,5 +31,13 @@ static struct context *_stp_runtime_get_context(void);
 
 #endif
 
+/* Define UBSAN related define if not defined by linux headers */
+#ifndef __counted_by
+#if __has_attribute(__counted_by__)
+#define __counted_by(member) __attribute__((__counted_by__(member)))
+#else
+#define __counted_by(member)
+#endif
+#endif
 
 #endif /* _RUNTIME_H_ */
