@@ -18,7 +18,11 @@
 #ifdef STP_NEED_LINE_DATA
 #include "unwind/unwind.h"
 #endif
+#ifdef STAPCONF_LINUX_UNALIGNED_H
+#include <linux/unaligned.h>
+#else
 #include <asm/unaligned.h>
+#endif
 #include <asm/uaccess.h>
 #include <linux/list.h>
 #include <linux/module.h>
@@ -1180,14 +1184,15 @@ unsigned long kallsyms_lookup_name (const char *name)
 }
 #endif
 
-#if defined(STAPCONF_KALLSYMS_ON_EACH_SYMBOL) && !defined(STAPCONF_KALLSYMS_ON_EACH_SYMBOL_EXPORTED)
+#if defined(STAPCONF_KALLSYMS_ON_EACH_SYMBOL)
+#if !defined(STAPCONF_KALLSYMS_ON_EACH_SYMBOL_EXPORTED)
 #ifndef KALLSYMS_H_INCLUDED
 #include <linux/kallsyms.h>
 #endif
 typedef typeof(&kallsyms_on_each_symbol) kallsyms_on_each_symbol_fn;
 
 // XXX Will be linked in place of the kernel's kallsyms_on_each_symbol:
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(6,4,0)
+#if defined(STAPCONF_KALLSYMS_6_4)
 int kallsyms_on_each_symbol(int (*fn)(void *, const char *,
 				      unsigned long),
                             void *data)
@@ -1209,18 +1214,19 @@ int kallsyms_on_each_symbol(int (*fn)(void *, const char *, struct module *,
         _stp_error("BUG: attempting to use unavailable kallsyms_on_each_symbol!!\n");
         return 0;
 }
+#endif // !defined(STAPCONF_KALLSYMS_ON_EACH_SYMBOL_EXPORTED)
 
+// XXX module_kallsyms_on_each_symbol has never been exported
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(5,12,0)
 typedef typeof(&module_kallsyms_on_each_symbol) module_kallsyms_on_each_symbol_fn;
 
-// XXX Will be linked in place of the kernel's module_kallsyms_on_each_symbol:
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(6,4,0)
+#if defined(STAPCONF_KALLSYMS_6_4)
 int module_kallsyms_on_each_symbol(const char *modname,
                                    int (*fn)(void *, const char *,
 				      unsigned long),
                             void *data)
 #else
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(6,3,0)
+#if defined(STAPCONF_KALLSYMS_6_3)
 int module_kallsyms_on_each_symbol(const char *modname,
 				   int (*fn)(void *, const char *, struct module *,
 				      unsigned long),
@@ -1235,7 +1241,7 @@ int module_kallsyms_on_each_symbol(int (*fn)(void *, const char *, struct module
         /* First, try to use a kallsyms_lookup_name address passed to us
            through the relocation mechanism. */
         if (_stp_module_kallsyms_on_each_symbol != NULL)
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(6,3,0)
+#if defined(STAPCONF_KALLSYMS_6_3) || defined(STAPCONF_KALLSYMS_6_4)
 		return ibt_wrapper(int,
 				   (* (module_kallsyms_on_each_symbol_fn)_stp_module_kallsyms_on_each_symbol)(modname, fn, data));
 #else
