@@ -262,7 +262,18 @@ static void _stp_handle_start(struct _stp_msg_start *st)
                 rcu_read_unlock();
 #endif
 
-		st->res = systemtap_module_init();
+#ifdef STAP_MODULE_INIT_HOOK
+		st->res = STAP_MODULE_INIT_HOOK();
+		if (st->res != 0) {
+			_stp_error ("Failed to run STAP_MODULE_INIT_HOOK "
+				    "(%d)\n", st->res);
+		} else {
+#else
+		{
+#endif
+			st->res = systemtap_module_init();
+		}
+
 		if (st->res == 0) {
 			_stp_probes_started = 1;
 
@@ -372,6 +383,12 @@ NEW_KALLSYMS_SYM(free_nsproxy)
 #if !defined(STAPCONF_UPROBE_UNREGISTER_EXPORTED)
         kallsyms_uprobe_unregister = (void*) kallsyms_lookup_name ("uprobe_unregister");
         cond_resched();  /* kallsyms_lookup_name is expensive; avoid blocking here */
+#if defined(STAPCONF_PR32194_UPROBE_REGISTER_UNREGISTER)
+        if (kallsyms_uprobe_unregister == NULL) {
+                kallsyms_uprobe_unregister = (void*) kallsyms_lookup_name ("uprobe_unregister_nosync");
+                cond_resched();  /* kallsyms_lookup_name is expensive; avoid blocking here */
+        }
+#endif
         if (kallsyms_uprobe_unregister == NULL) {
                 kallsyms_uprobe_unregister = (void*) kallsyms_lookup_name ("unregister_uprobe");
                 cond_resched();  /* kallsyms_lookup_name is expensive; avoid blocking here */
@@ -580,6 +597,9 @@ static void _stp_transport_close(void)
 	_stp_unregister_ctl_channel();
 	_stp_print_cleanup(); /* Requires the transport, so free this first */
 	_stp_transport_fs_close();
+#ifdef STAP_MODULE_EXIT_HOOK
+	STAP_MODULE_EXIT_HOOK();
+#endif
 	_stp_mem_debug_done();
 
 	dbug_trans(1, "---- CLOSED ----\n");
