@@ -29,6 +29,7 @@
 #include <functional> 
 #include <cctype>
 #include <locale>
+#include <memory>
 
 extern "C" {
 #include <elf.h>
@@ -44,6 +45,8 @@ extern "C" {
 #include <unistd.h>
 #include <regex.h>
 #include <stdarg.h>
+#include <libgen.h>
+#include <pwd.h>
 
 #ifdef HAVE_LIBDEBUGINFOD
 #include <elfutils/debuginfod.h>
@@ -189,7 +192,9 @@ copy_file(const string& src, const string& dest, bool verbose)
 
 error:
   cerr << _F("Copy failed (\"%s\" to \"%s\"): %s", src.c_str(),
-             dest.c_str(), strerror(errno)) << endl;
+             dest.c_str(), strerror(errno))
+       << ((getuid() != 0) ?
+           _F(". Your uid=%d.", getuid()) : "") << endl;
   return false;
 }
 
@@ -222,6 +227,7 @@ create_dir(const char *dir, int mode)
   for (unsigned ix = 0; ix < limit; ++ix)
     {
       path += components[ix] + '/';
+      umask(0); mode=0777;
       if (mkdir(path.c_str (), mode) != 0 && errno != EEXIST)
 	return 1;
     }
@@ -250,6 +256,20 @@ remove_file_or_dir (const char *name)
 }
 
 
+void 
+split_path (string &path, string &directory, string &entry)
+{
+  char *dirc, *basec, *bname, *dname;
+
+  dirc = strdupa (path.c_str());
+  basec = strdupa (path.c_str());
+  dname = dirname (dirc);
+  bname = basename (basec);
+  directory = dname;
+  entry = bname;
+}
+
+
 int
 appendenv (const char *env_name, const string source)
 {
@@ -492,7 +512,7 @@ follow_link(const string& name, const string& sysroot)
 	{
 	  string fname1 = sysroot + linkname;
 	  const char *f1 = fname1.c_str();
-	  if (access(f1, X_OK) == 0
+	  if (access(f1, R_OK) == 0
 	      && stat(f1, &st) == 0
 	      && S_ISREG(st.st_mode))
 	    {
@@ -567,7 +587,7 @@ string find_executable(const string& name, const string& sysroot,
 
       const char *f = fname.c_str();
 
-      if (access(f, X_OK) == 0
+      if (access(f, R_OK) == 0
           && stat(f, &st) == 0
           && S_ISREG(st.st_mode))
         {
@@ -583,12 +603,13 @@ string find_executable(const string& name, const string& sysroot,
       if (retpath == "")
         {
           // Query debuginfod for the executable.
-          debuginfod_client *client = debuginfod_begin();
-
-          if (client != NULL)
+          static unique_ptr <debuginfod_client, void (*)(debuginfod_client*)>
+            client (debuginfod_begin(), &debuginfod_end);
+          
+          if (client.get() != NULL)
             {
               char *p;
-              int fd = debuginfod_find_executable(client,
+              int fd = debuginfod_find_executable(client.get(),
                                                   (const unsigned char*)(name.c_str()),
                                                   0, &p);
               if (fd >= 0)
@@ -597,7 +618,6 @@ string find_executable(const string& name, const string& sysroot,
                   free(p);
                   close(fd);
                 }
-              debuginfod_end(client);
             }
         }
 #endif /* HAVE_LIBDEBUGINFOD */
@@ -634,7 +654,7 @@ string find_executable(const string& name, const string& sysroot,
               const char *f = fname.c_str();
 
               // Look for a normal executable file.
-              if (access(f, X_OK) == 0
+              if (access(f, R_OK) == 0
                   && stat(f, &st) == 0
                   && S_ISREG(st.st_mode))
                 {
@@ -1285,10 +1305,9 @@ octal_character (unsigned c)
   return '0' + c % 8;
 }
 
-string
-escaped_character (unsigned c)
+static inline void
+escaped_character_impl (unsigned c, ostringstream &o)
 {
-  ostringstream o;
   int oc = (int)c;
 
   switch (oc)
@@ -1346,18 +1365,28 @@ escaped_character (unsigned c)
             << octal_character(oc);
         }
     }
+}
+
+string
+escaped_character (unsigned c)
+{
+  ostringstream o;
+
+  escaped_character_impl(c, o);
+
   return o.str();
 }
 
 string
 escaped_literal_string (const string& str)
 {
-  string op;
+  ostringstream o;
+
   for (unsigned i = 0; i < str.size (); i++)
     {
-      op += escaped_character((unsigned)str[i]);
+      escaped_character_impl((unsigned char)str[i], o);
     }
-  return op;
+  return o.str();
 }
 
 string
@@ -1383,6 +1412,7 @@ normalize_machine(const string& machine)
   else if (machine == "sa110") return "arm";
   else if (machine == "s390x") return "s390";
   else if (machine == "aarch64") return "arm64";
+  else if (machine == "riscv64") return "riscv";
   else if (machine.substr(0,3) == "ppc") return "powerpc";
   else if (machine.substr(0,4) == "mips") return "mips";
   else if (machine.substr(0,3) == "sh2") return "sh";
@@ -1479,7 +1509,10 @@ is_valid_pid (pid_t pid, string& err_msg)
     }
   else if (kill(pid, 0) == -1)
     {
-      err_msg = _F("cannot probe pid %d: %s", pid, strerror(errno));
+      if (getuid() != 0)
+        err_msg = _F("cannot probe pid %d: %s.  Your uid=%d.", pid, strerror(errno), getuid());
+      else
+        err_msg = _F("cannot probe pid %d: %s", pid, strerror(errno));
       return false;
     }
   return true;
@@ -1542,7 +1575,7 @@ levenshtein_suggest(const string& target,        // string to match against
 
       // Approximate levenshtein by size-difference only; real score
       // is at least this high
-      unsigned min_score = labs(target.size() - it->size());
+      unsigned min_score = abs(static_cast<signed>(target.size()) - static_cast<signed>(it->size()));
 
       if (min_score > threshold) // min-score too high for threshold
         continue;
@@ -1755,21 +1788,24 @@ flush_to_stream (const string &fname, ostream &o)
   return 1; // Failure
 }
 
+int
+not_isspace(unsigned char c)
+{
+  return !std::isspace(c);
+}
+
 // trim from start (in place)
 void
 ltrim(std::string &s)
 {
-  s.erase(s.begin(),
-	  std::find_if(s.begin(), s.end(),
-		       std::not1(std::ptr_fun<int, int>(std::isspace))));
+  s.erase(s.begin(), std::find_if(s.begin(), s.end(), not_isspace));
 }
 
 // trim from end (in place)
 void
 rtrim(std::string &s)
 {
-  s.erase(std::find_if(s.rbegin(), s.rend(),
-	  std::not1(std::ptr_fun<int, int>(std::isspace))).base(), s.end());
+  s.erase(std::find_if(s.rbegin(), s.rend(), not_isspace).base(), s.end());
 }
 
 // trim from both ends (in place)
@@ -1866,4 +1902,31 @@ get_distro_info(vector<string> &info)
     return false;
 }
 
+// PR30321: Privilege separation
+int
+run_unprivileged(const std::string& build_as, uid_t build_as_uid, gid_t build_as_gid, int verbosity)
+{
+    if (build_as == "")
+      return EXIT_SUCCESS;
+
+    int ret;
+    ret = setregid(build_as_gid, build_as_gid);
+    if (ret != 0) {
+        clog << "ERROR: setregid() failed" << endl;
+        clog << strerror (errno) << endl;
+        return EXIT_FAILURE;
+    }
+    ret = setreuid(build_as_uid, build_as_uid);
+    if (ret != 0) {
+        clog << "ERROR: setreuid() failed" << endl;
+        clog << strerror (errno) << endl;
+        return EXIT_FAILURE;
+    }
+    if (verbosity > 2)
+      cout << _F("Running passes 1-4 using user \"%s\" userid \"%d\" group id \"%d\"",
+                  build_as.c_str(), build_as_uid, build_as_gid) << "<<<" << endl;
+    return EXIT_SUCCESS;
+}
+
+
 /* vim: set sw=2 ts=8 cino=>4,n-2,{2,^-2,t0,(0,u0,w1,M1 : */
